from __future__ import print_function

import click
import fileinput
import os
import shutil
import subprocess
import sys
import tempfile
import textwrap
import time

from obsctl.lib import obsc
from obsctl.cli import pass_context

try:
    import osc.conf
    import osc.core
    import osc.build
except ImportError:
    print("The Open Build Service Commander is not installed!")
    sys.exit(2)


@click.command('scratchbuild',
               help='Executes a local scratch build for OBS projects '
                    'given a git repo',
               short_help='Executes local OBS scratch build')
@click.option('--obs-project', '-s', help='Open Build Service project space',
              required=True)
@click.option('--obs-package', '-p', help='Open Build Service project package',
              required=True)
@click.option('--build-distribution', '-d', default='xUbuntu_16.04',
              help='Distribution to build project in')
@click.option('--commit-hash', '-c', help='Source repository release commit',
              required=True)
@click.option('--repo-reltag', '-t', help='Source repository release tag',
              required=True)
@click.option('--build-branch', '-B', help='Source repository build branch')
@click.option('--commit-time', '-T', help='Source repository commit time',
              default=time.strftime('%Y%m%d%H%M%S'))
@pass_context
def cli(ctx, obs_project, obs_package,
        build_distribution, commit_hash, repo_reltag, build_branch,
        commit_time):
    # TODO: Simplify this, there must be some way to capture all arguments in
    #       a map or split things out more logically, this is for the sake of
    #       time.
    args = {
            'obs_server_addr': ctx.obs_server,
            'obs_auth_user': ctx.obs_user,
            'obs_auth_pass': ctx.obs_pass,
            'obs_project': obs_project,
            'obs_package': obs_package,
            'build_dist': build_distribution,
            'build_commit_hash': commit_hash,
            'repo_tag': repo_reltag,
            'build_branch': build_branch,
            'commit_time': commit_time
            }
    main(args)


def get_obs_newpkg_conf(args):
    print(args)
    return """
    <package name="{package}" project="{project}">
      <title>{package} for {project}</title>
      <description>{package} for {project} created by {user}.</description>
    </package>
    """.format(package=args['obs_package'], project=args['obs_project'],
               user=args['obs_auth_user'])


def checkout_obs_project(args, project_checkout_dir):
    # check if project does exist
    try:
        osc.core.show_project_meta(args['obs_server_addr'],
                                   args['obs_project'])

        return osc.core.Project.init_project(args['obs_server_addr'],
                                             project_checkout_dir,
                                             args['obs_project'])
    except Exception as e:
        print('Error when checking out project: {}'.format(e))
        exit()


def create_obs_package(args, project, package_checkout_dir):
    try:
        return osc.core.Package.init_package(args['obs_server_addr'],
                                             args['obs_project'],
                                             args['obs_package'],
                                             package_checkout_dir)
    except Exception as e:
        print("failed to create temporary package.")
        print("Exception: {}".format(e))
        exit()


def make_project_tarball(args, tag, target_dir):
    tarball_dir = "{}-{}".format(args['obs_package'], tag)
    shutil.copytree('.', tarball_dir, symlinks=True,
                    ignore=shutil.ignore_patterns('scratchbuild*'))
    repo_tarball = shutil.make_archive(base_name=os.path.join(target_dir,
                                                              tarball_dir),
                                       base_dir=tarball_dir,
                                       format="gztar")
    shutil.rmtree(tarball_dir)

    return repo_tarball


def execute(cmd, dir):
    curr_dir = os.getcwd()
    os.chdir(dir)
    popen = subprocess.Popen(cmd, stdout=subprocess.PIPE,
                             universal_newlines=True, shell=True)
    for stdout_line in iter(popen.stdout.readline, ""):
        yield stdout_line
    popen.stdout.close()
    os.chdir(curr_dir)
    return_code = popen.wait()

    if return_code:
        raise subprocess.CalledProcessError(return_code, cmd)


def main(args):
    oscconf = obsc.configure(args['obs_server_addr'],
                             args['obs_auth_user'],
                             args['obs_auth_pass'],
                             insecure_connection=True)

    prj_checkout_dir = tempfile.mkdtemp(suffix="{}"
                                        .format(args['obs_project']))
    pkg_checkout_dir = tempfile.mkdtemp(suffix="{}"
                                        .format(args['obs_package']))
    obs_project = checkout_obs_project(args, prj_checkout_dir)
    obs_package = create_obs_package(args, obs_project, pkg_checkout_dir)

    # Configure version form
    if args['build_branch'] is not None:
        branch = args['build_branch'].replace('-', '').replace('_', '').lower()
        post_release_tag = "{0}+{1}{2}.{3}.{4}".format(
                                                    args['repo_tag'],
                                                    "git",
                                                    args['commit_time'],
                                                    args['build_commit_hash'],
                                                    branch
                                                )
    else:
        post_release_tag = "{0}+{1}{2}.{3}".format(args['repo_tag'],
                                                   "git",
                                                   args['commit_time'],
                                                   args['build_commit_hash'])

    repo_tarball = make_project_tarball(args, post_release_tag,
                                        pkg_checkout_dir)

    obs_package_service = """
    <services>
      <service name="extract_file">
        <param name="archive">{package}-{tag}.tar.gz</param>
        <param name="files">*{package}*/dist/{package}.spec</param>
      </service>
    </services>
    """.format(package=args['obs_package'], project=args['obs_project'],
               tag=post_release_tag)

    obs_package_service_file = open(pkg_checkout_dir + "/_service", 'w')
    obs_package_service_file.write(obs_package_service)
    obs_package_service_file.close()

    # A bit of a workaround but it works :)
    obs_package.filenamelist.append("_service")

    obs_package.run_source_services()

    # Now replace version and release numbers
    service_file = ('{package_dir}/_service:extract_file:{package}.spec'
                    .format(package_dir=obs_package.absdir,
                            package=args['obs_package']))
    try:
        for line in fileinput.input(service_file, inplace=True):
            if "Version:" in line:
                print("Version:        {tag}".format(tag=post_release_tag))
            else:
                print(line, end="")
    finally:
        fileinput.close()

    """ Here RIP the attempt to use osc api """
    """
    opts = {
                'no_verify': True,
                'userootforbuild': False,
                'trust_all_projects': True,
                'jobs': '4',
                'root': os.path.join(os.getcwd(),
                                     'scratchbuild-' + args.build_dist)
            }
    build_args = [args.build_dist, 'x86_64',
                  '{}/{}.spec'.format(obs_package.absdir, args.obs_package)]

    osc.build.main(args.obs_server_addr, opts, build_args)
    """
    build_command = ("osc build --jobs=4 --trust-all-project --no-verify "
                     "--noservice --root={path}/scratchbuild-{dist} {dist}"
                     .format(path=os.getcwd(), dist=args['build_dist']))
    for path in execute(build_command, obs_package.absdir):
        print(path, end="")

    # Clean up
    if os.path.isfile("{}-{}.tar.gz".format(args['obs_package'],
                                            post_release_tag)):
        os.remove("{}-{}.tar.gz".format(args['obs_package'], post_release_tag))
    oscconf.close()

    print("Scratchbuild Completed Successfully!!")
